public class RubiksCube {

	private Piece[][][] cube ;

	public RubiksCube () {
		cube = new Piece[3][3][3] ;

		int x, y, z ;
		for (x = 0 ; x < 3 ; x++)
			for (y = 0 ; y < 3 ; y ++)
				for (z = 0 ; z < 3 ; z ++)
					cube[x][y][z] = new Piece() ;

		setInitialState () ;
	}


	public void setInitialState () {
		int x, y, z ;

		// FRONT face :
		z = 0 ;
		for (x = 0 ; x < 3 ; x++)
			for (y = 0 ; y < 3 ; y++)
				cube[x][y][z].setColour(Face.FRONT, Colour.RED) ;

		// TOP face :
		y = 2 ;
		for (x = 0 ; x < 3 ; x++)
			for (z = 0 ; z < 3 ; z++)
				cube[x][y][z].setColour(Face.TOP,   Colour.BLUE) ;

		// RIGHT face :
		x = 2 ;
		for (y = 0 ; y < 3 ; y++)
			for (z = 0 ; z < 3 ; z++)
				cube[x][y][z].setColour(Face.RIGHT, Colour.YELLOW) ;

		// LEFT  face :
		x = 0 ;
		for (y = 0 ; y < 3 ; y++)
			for (z = 0 ; z < 3 ; z++)
				cube[x][y][z].setColour(Face.LEFT,  Colour.WHITE) ;

		// DOWN  face :
		y = 0 ;
		for (x = 0 ; x < 3 ; x++)
			for (z = 0 ; z < 3 ; z++)
				cube[x][y][z].setColour(Face.DOWN,  Colour.GREEN) ;

		// BACK face :
		z = 2 ;
		for (x = 0 ; x < 3 ; x++)
			for (y = 0 ; y < 3 ; y++)
				cube[x][y][z].setColour(Face.BACK, Colour.ORANGE) ;
	}

	private void rotateClockwizeAround(int axis, int ordinate) {
					// for the cube, the axes are seen as crossing
					// at [1][1][1] (not [0][0][0]).
					// The ordinate is given with respect to this base.

		Piece safePiece = null ;

		switch (axis) {

			case Axis.XAXIS :
						// (along positive direction of X Axis)
				int xfixed = ordinate + 1 ;	// following the change of basis
																		// (the center of the cube is at [1][1][1])
		
				// rotating the corners:
				safePiece          = cube[xfixed][2][0] ;
				cube[xfixed][2][0] = cube[xfixed][2][2] ;
				cube[xfixed][2][2] = cube[xfixed][0][2] ;
				cube[xfixed][0][2] = cube[xfixed][0][0] ;
				cube[xfixed][0][0] = safePiece ;
		
				// rotating the edges:
				safePiece          = cube[xfixed][2][1] ;
				cube[xfixed][2][1] = cube[xfixed][1][2] ;
				cube[xfixed][1][2] = cube[xfixed][0][1] ;
				cube[xfixed][0][1] = cube[xfixed][1][0] ;
				cube[xfixed][1][0] = safePiece ;
		
				// rotating the pieces onto themselves :
				for (int y = 0 ; y < 3 ; y++)
					for (int z = 0 ; z < 3 ; z++)
						cube[xfixed][y][z].rotateClockwizeAround(Axis.XAXIS) ;
		
				break ;
		
			case Axis.YAXIS :
							// (along positive direction of Y Axis)
				int yfixed = ordinate + 1 ;
		
				// rotating the corners:
				safePiece          = cube[0][yfixed][2] ;
				cube[0][yfixed][2] = cube[2][yfixed][2] ;
				cube[2][yfixed][2] = cube[2][yfixed][0] ;
				cube[2][yfixed][0] = cube[0][yfixed][0] ;
				cube[0][yfixed][0] = safePiece ;
		
				// rotating the edges:
				safePiece          = cube[1][yfixed][2] ;
				cube[1][yfixed][2] = cube[2][yfixed][1] ;
				cube[2][yfixed][1] = cube[1][yfixed][0] ;
				cube[1][yfixed][0] = cube[0][yfixed][1] ;
				cube[0][yfixed][1] = safePiece ;
		
				// rotating the pieces onto themselves :
				for (int x = 0 ; x < 3 ; x++)
					for (int z = 0 ; z < 3 ; z++)
						cube[x][yfixed][z].rotateClockwizeAround(Axis.YAXIS) ;
		
				break ;
		
			case Axis.ZAXIS :
							// (along positive direction of Z Axis)
				int zfixed = ordinate + 1 ;
		
				// rotating the corners:
				safePiece          = cube[0][2][zfixed] ;
				cube[0][2][zfixed] = cube[0][0][zfixed] ;
				cube[0][0][zfixed] = cube[2][0][zfixed] ;
				cube[2][0][zfixed] = cube[2][2][zfixed] ;
				cube[2][2][zfixed] = safePiece ;
		
				// rotating the edges:
				safePiece          = cube[1][2][zfixed] ;
				cube[1][2][zfixed] = cube[0][1][zfixed] ;
				cube[0][1][zfixed] = cube[1][0][zfixed] ;
				cube[1][0][zfixed] = cube[2][1][zfixed] ;
				cube[2][1][zfixed] = safePiece ;
		
				// rotating the pieces onto themselves :
				for (int x = 0 ; x < 3 ; x++)
					for (int y = 0 ; y < 3 ; y++)
						cube[x][y][zfixed].rotateClockwizeAround(Axis.ZAXIS) ;
		
				break ;
	
			default : System.out.println("bug34\n") ;
		}
	}
		

	public void T() {
						// rotating TOP face clockwize
		for (int i = 0 ; i < 3 ; i++)
			rotateClockwizeAround(Axis.YAXIS, 1) ;
	}

	public void D() {
						// rotating DOWN face clockwize
		rotateClockwizeAround(Axis.YAXIS,  -1) ;
	}

	public void F() {
						// rotating FRONT face clockwize
		rotateClockwizeAround(Axis.ZAXIS,  -1) ;
	}

	public void B() {
						// rotating BACK face clockwize
		for (int i = 0 ; i < 3 ; i++)
			rotateClockwizeAround(Axis.ZAXIS, 1) ;
	}

	public void L() {
						// rotating LEFT face clockwize
		rotateClockwizeAround(Axis.XAXIS,  -1) ;
	}
	public void R() {
						// rotating RIGHT face clockwize
		for (int i = 0 ; i < 3 ; i++)
			rotateClockwizeAround(Axis.XAXIS, 1) ;
	}


	public boolean isInInitialState () {
		boolean toBeReturned = true ;

		int x, y, z ;

		// test RED on FRONT face:
		z = 0 ;
		for (x = 0 ; x < 3 ; x++)
			for (y = 0 ; y < 3 ; y++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.FRONT).equals(Colour.RED) ;

		// test ORANGE on BACK face:
		z = 2 ;
		for (x = 0 ; x < 3 ; x++)
			for (y = 0 ; y < 3 ; y++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.BACK).equals(Colour.ORANGE) ;

		// test BLUE on TOP face:
		y = 2 ;
		for (x = 0 ; x < 3 ; x++)
			for (z = 0 ; z < 3 ; z++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.TOP).equals(Colour.BLUE) ;

		// test GREEN on DOWN face:
		y = 0 ;
		for (x = 0 ; x < 3 ; x++)
			for (z = 0 ; z < 3 ; z++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.DOWN).equals(Colour.GREEN) ;

		// test WHITE on LEFT face:
		x = 0 ;
		for (y = 0 ; y < 3 ; y++)
			for (z = 0 ; z < 3 ; z++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.LEFT).equals(Colour.WHITE) ;

		// test YELLOW on RIGHT face:
		x = 2 ;
		for (y = 0 ; y < 3 ; y++)
			for (z = 0 ; z < 3 ; z++)
				toBeReturned = toBeReturned &&
					cube[x][y][z].getColour(Face.RIGHT).equals(Colour.YELLOW) ;

		return toBeReturned ;
	}

	public String toString() {
		String toBeReturned = "" ;
		int x, y, z;
		int l;

		// BACK face :
		z = 2 ;
		for (y = 0 ; y < 3 ; y++)
		{
			toBeReturned += "   |" ;
			for (x = 0 ; x < 3 ; x++)
				toBeReturned += cube[x][y][z].toString(Face.BACK) ;
			toBeReturned += "|\n" ;
		}

		toBeReturned += "    --- \n" ;
		// TOP   face :
		y = 2 ;
		for (z = 2 ; z >= 0 ; z--)
		{
			toBeReturned += "   |" ;
			for (x = 0 ; x < 3 ; x++)
				toBeReturned += cube[x][y][z].toString(Face.TOP) ;
			toBeReturned += "|\n" ;
		}

		toBeReturned += "    --- \n" ;

		// LEFT, FRONT and RIGHT faces :
		int xl, zl ;
		int xr, zr ;
		xl = 0 ;
		xr = 2 ;

		z = 0 ;
		for (y = 2 ; y >= 0 ; y--)
		{
			for (zl = 2 ; zl >= 0 ; zl--)
				toBeReturned += cube[xl][y][zl].toString(Face.LEFT) ;

			toBeReturned += "|" ;

			for (x = 0 ; x < 3 ; x++)
				toBeReturned += cube[x][y][z].toString(Face.FRONT) ;

			toBeReturned += "|" ;

			for (zr = 0 ; zr < 3 ; zr++)
				toBeReturned += cube[xr][y][zr].toString(Face.RIGHT) ;

			toBeReturned += "\n" ;
		}

		toBeReturned += "    --- \n" ;

		// BACK face :
		y = 0 ;
		for (z = 0 ; z < 3 ; z++)
		{
			toBeReturned += "   |" ;
			for (x = 0 ; x < 3 ; x++)
				toBeReturned += cube[x][y][z].toString(Face.DOWN) ;
			toBeReturned += "|\n" ;
		}

		toBeReturned += "\n" ;

		return toBeReturned ;
	}
}
