import java.util.Hashtable ;
import java.util.Map ;
import java.util.Iterator ;

import java.util.Enumeration ;
import java.util.HashSet ;


/** Matrices creuses de "double". **/


public class Matrice {

  private int nbLignesOfficiel ;
  private int nbColonnesOfficiel ;

  private Hashtable lignes ;
      /** la matrice est representee par un dictionnaire de dictionnaires.
      Le dictionnaire principal a les indices des lignes en cle,
      les dictionnaires secondaires ont les indices de colonne pour cle.
      **/


  public static Matrice creerMatriceIdentite(int n) {
    Matrice aRetourner = new Matrice (n, n) ;
    int i, j ;
    for (i = 1 ; i <= n ; i++)
      aRetourner.set(i, i, 1.0) ;
    return aRetourner ;
  }


  /** matrice creee avec des zeros partout **/
  public Matrice (int n, int m) {
    nbLignesOfficiel = n ;
    nbColonnesOfficiel = m ;
    lignes = new Hashtable() ;
  }



  /** retourne un pointeur sur l'objet Double emballant le double recherche ;
  la valeur null si le double est 0.0 et n'est pas stocke. **/
  private Double getEmballage(int ii, int jj) {
    Double d ;
    Hashtable ligne = (Hashtable)lignes.get(new Integer(ii)) ;
    if (ligne != null)
      d = (Double)ligne.get(new Integer(jj)) ;
      // peut etre "null"
    else
      d = null ;
    return d ;
  }

  private boolean existenceStockageAssocie(int ii, int jj) {
    return (getEmballage(ii, jj) != null) ;
  }

  /** recuperer la valeur d'un element de la matrice **/
  public double get(int ii, int jj) {
    double aRetourner ;

    Double emballageCherche = getEmballage(ii, jj) ;
    if (emballageCherche != null)
      aRetourner = emballageCherche.doubleValue() ;
    else
      // pas d'emballege : cela signifie que l'element vaut 0.0.
      aRetourner = 0.0 ;

    return aRetourner ;
  }


  /** fixer la valeur d'un element de la matrice **/
  public void set(int ii, int jj, double xx) {
    Hashtable ligne = (Hashtable)lignes.get(new Integer(ii)) ;
    
    // si deja une valeur stockee a ces indices : on l'ecrase si xx != 0.0 ;
    // on efface simplement l'ancien stockage de valeur, si xx == 0.0.
    if (ligne != null)
    {
      if (xx != 0.0)
        ligne.put(new Integer(jj), new Double(xx)) ;
      else
        if (ligne.containsKey(new Integer(jj)))
          ligne.remove(new Integer(jj)) ;
    }
    // si aucune valeur encore stockee a cet indice : on
    // cree le stockage, si xx != 0.0.
    else
      if (xx != 0.0)
      {
        ligne = new Hashtable() ;
        ligne.put(new Integer(jj), new Double(xx)) ;
        lignes.put(new Integer(ii), ligne) ;
      }
  }

  public int nbLignes () { return nbLignesOfficiel ; }

  public int nbColonnes () { return nbColonnesOfficiel ; }

  public Matrice mult(double factor) {
    Matrice aRetourner = new Matrice (nbLignes(), nbColonnes()) ;

    int i, j ;
    double xx ;
    Iterator iterEntriesI = lignes.entrySet().iterator() ;
    while (iterEntriesI.hasNext())
    {
      Map.Entry mapI = (Map.Entry)iterEntriesI.next() ;
      i = ((Integer)mapI.getKey()).intValue() ;
      Iterator iterEntriesJ = ((Hashtable)mapI.getValue()).entrySet().iterator() ;
      while (iterEntriesJ.hasNext())
      {
        Map.Entry mapJ = (Map.Entry)iterEntriesJ.next() ;
        j = ((Integer)mapJ.getKey()).intValue() ;
        xx = ((Double)mapJ.getValue()).doubleValue() ;

        aRetourner.set(i,  j, xx * factor) ;
      }
    }
    return aRetourner ;
  }


  private HashSet unionDesCles(Hashtable a, Hashtable b) {
    HashSet unionDesCles = new HashSet(a.keySet()) ;
    Enumeration clesDeBEnum = b.keys() ;
    while (clesDeBEnum.hasMoreElements())
      unionDesCles.add(clesDeBEnum.nextElement()) ;
    return unionDesCles ;
  }


  public Matrice plus(Matrice b) {
    Matrice aRetourner ;
    if ((nbLignes() != b.nbLignes()) || (nbColonnes() != b.nbColonnes()))
    {
      System.out.println(
  "Ces matrices n'ont pas la meme taille. On ne peut pas les additionner.") ;
      aRetourner = null ;
    }
    else
    {
      aRetourner = new Matrice(nbLignes(), nbColonnes()) ;

      int i, j ;

      Iterator lUnionIIter, lUnionJIter ;
  
      // boucle 1 : iteration sur l'union des indices "i" des
      // lignes non zero-partout des deux matrices :
      lUnionIIter = unionDesCles(this.lignes, b.lignes).iterator() ;
      while (lUnionIIter.hasNext())
      {

        // boucle 2 : iteration sur l'union des indices "j" des deux matrices :
        Integer ICourant = (Integer)lUnionIIter.next() ;
        i = ICourant.intValue() ;

        Hashtable ligneA = (Hashtable)this.lignes.get(ICourant) ;
        Hashtable ligneB = (Hashtable)b.lignes.get(ICourant) ;
        if (ligneA == null)
          ligneA = new Hashtable() ;
        if (ligneB == null)
          ligneB = new Hashtable() ;

        lUnionJIter = unionDesCles(ligneA, ligneB).iterator() ;
        while (lUnionJIter.hasNext())
        {
          j = ((Integer)lUnionJIter.next()).intValue() ;

          // le calcul et le stockage des additions :
          aRetourner.set(i, j, this.get(i, j) + b.get(i, j)) ;
        }
      }
    }

    return aRetourner ;
  }

  public Matrice mult(Matrice b) {
    Matrice aRetourner ;
    if (nbColonnes() != b.nbLignes())
    {
      System.out.println(
"Dimensions incompatibles pour la multiplication. Ces matrices ne peuvent pas etre multipliees.") ;
      aRetourner = null ;
    }
    else
    {
      /* idee generale : on ne fait des produits que sur deux valeurs non nulles.

      Algorithme :
      Soit c le nome de la matrice a construire (on veut c = this * b) :
      pour toutes les valeurs non nulles this_ij de this,
      trouver toutes les valeurs non nulles b_jl, l = 1, ... n ;
      puis faire le produit this_ij * b_jl, et l'ajouter a c_il, que l'on aura initialise a 0.0.
      */

      // 0) matrice c creee et initialisee avec des zeros partout.
      aRetourner = new Matrice(nbLignes(), b.nbColonnes()) ;

      // 1) trouver toutes les valeurs non nulles this_ij de this, avec leurs indices i et j :
      int i, l ;
      Integer J ;
      double xx, yy, previous ;

      Map.Entry mapI, mapJ, mapL ;
      Iterator iterEntriesI, iterEntriesJ, iterEntriesOfLineJofB ;


      // boucle sur les I de this :

      iterEntriesI = lignes.entrySet().iterator() ;

      while (iterEntriesI.hasNext())
      {
        mapI = (Map.Entry)iterEntriesI.next() ;
        i = ((Integer)mapI.getKey()).intValue() ;


        // boucle sur les J de la ligne I de this :

        iterEntriesJ = ((Hashtable)mapI.getValue()).entrySet().iterator() ;

        while (iterEntriesJ.hasNext())
        {
          mapJ = (Map.Entry)iterEntriesJ.next() ;
          J = (Integer)mapJ.getKey() ;
          xx = ((Double)mapJ.getValue()).doubleValue() ;

          // 2) pour ce j, trouver toutes les valeurs non nulles b_jl, 


          // boucle sur les L non nuls de la ligne J de b (s'il y en a) :

          if (b.lignes.containsKey(J))
          {
            iterEntriesOfLineJofB = ((Hashtable)b.lignes.get(J)).entrySet().iterator() ;

            while (iterEntriesOfLineJofB.hasNext())
            {
              mapL = (Map.Entry)iterEntriesOfLineJofB.next() ;
              l = ((Integer)mapL.getKey()).intValue() ;
              yy = ((Double)mapL.getValue()).doubleValue() ;
              
              // 3) le produit et l'addition :
              previous =  aRetourner.get(i, l) ;  
              aRetourner.set(i, l, previous + (xx * yy)) ;
            }
          }
        }
      }
    }

    return aRetourner ;
  }

  public String toString () {
    String aRetourner = "" ;

    int i, j ;
    Iterator iterEntriesI = lignes.entrySet().iterator() ;
    while (iterEntriesI.hasNext())
    {
      Map.Entry mapI = (Map.Entry)iterEntriesI.next() ;
      i = ((Integer)mapI.getKey()).intValue() ;
      Iterator iterEntriesJ =
            ((Hashtable)mapI.getValue()).entrySet().iterator() ;
      while (iterEntriesJ.hasNext())
      {
        Map.Entry mapJ = (Map.Entry)iterEntriesJ.next() ;
        j = ((Integer)mapJ.getKey()).intValue() ;
        aRetourner +=   "A (" + i + ", " + j + ") : "
                      + ((Double)mapJ.getValue()).doubleValue() + "\n" ;
      }
    }
    return aRetourner ;
  }
}
